//
//  GlassboxAgent.h
//  Glassbox
//
//  Copyright © 2018 Glassbox. All rights reserved.
//
//
//
#import <Foundation/Foundation.h>
#import <UIKit/UIViewController.h>

@protocol GLASessionDelegate <NSObject>

@optional

/**
 Called when configuration is received successfully
 @param configuration The received configuration
 */
- (void)sessionStartedWithConfiguration:(NSDictionary *)configuration;

/**
 Called when failed receiving configuration
 */
- (void)sessionFailedWithError:(NSError *)error;

/**
 Called when configuration is received successfully but the session is not recorded, for example due to configured device filter
 @param reason Why the session is not recorded
 @param configuration The received configuration
 */
- (void)sessionExcludedWithReason:(NSString *)reason withConfiguration:(NSDictionary *)configuration;

@end


/**
 * Controls configuration environment mode.
 * 1. EXTERNAL - Default configuration mode.
 * 2. INTERNAL - use when configuration is hosted on a private server.
 */
typedef NS_ENUM(NSUInteger, GLAConfigurationType) {
    GLAConfigurationExternal = 0,
    GLAConfigurationInternal
};

@interface GLAStartupSettings :NSObject

+ (instancetype)settings;

@property NSString *reportUrl;

/**
 * Optional url to fetch sdk configuration from, if not set the report url is used.
 */
@property NSString *appConfigUrl;

@property NSString *appId;

@property NSData *certificate;

@property BOOL hybridMode;

@property GLAConfigurationType configuration;

@property BOOL omitMonitoringBeforeConfig;

@property BOOL disableNewSessionizing;

@end

/**
 * Controls touch position within a sensitive view.
 * 1. CENTER - Replace the touch location with the center of the view
 * 2. NONE - Removes the touch location completely
 * 3. DISPLAY - No changes (default)
 */
typedef NS_ENUM(NSUInteger, GLATouchMode) {
    GLA_CENTER = 0,
    GLA_NONE,
    GLA_DISPLAY
};

@interface GLAVisibilityFlags : NSObject

/**
 *  Default visibility flags
 */
+ (instancetype)flags;

/**
 * Create visibility flags from a dictionary
 */
+ (instancetype)flagsFromDictionary:(NSDictionary *)dictionary;

- (NSString *)description;

/**
 * Indicates if UIView elements with alpha property set to 0 should be masked.
 * Default is NO.
 */
@property BOOL maskOnAlpha;

/**
 * Indicates if UIView elements with isHidden property set to true should be masked.
 * Default is NO.
 */
@property BOOL maskOnHidden;

/**
 * Indicates if UIView which reside on a UIViewController which is not top view controller should be masked.
 * Default is YES.
 */
@property BOOL maskOnAllControllers;

/**
 * Hides all views with an equal tag to the provided view.
 */
@property BOOL maskByTag;

/**
 * Omit analytics info for touch event
 */
@property BOOL omitAnalytics;


@property GLATouchMode touchMode;

@end

/*!
 * @framework Glassbox
 * @brief Entry point to Glassbox SDK.
 * @discussion GlassboxAgent provide set of api which is accessible for user which
 * are working with this sdk.
 * No other api is visible in this framework.
 */
@interface GlassboxAgent : NSObject

/*!
 * @deprecated Please use the start:settings method
 * @function start
 * @brief Starts Glassbox Monitoring.
 * @param error
 *        if not nil, an error occurred, if the error domain is com.clarisite.mobile, then it's
 *        related to the framework it self.
 */
+ (BOOL)start:(NSError **)error __attribute__((deprecated("Please use start:settings method instead")));

/**
 Set a delegate to receive callback on session events such as when configuration is received from the server or when it fails
 @param delegate which implements protocol GLASessionDelegate
 */
+ (void)setSessionDelegate:(id <GLASessionDelegate>)delegate;

/**
 * @deprecated Please use the start:settings method
 * @function start:serverUrl:appId:userData:error
 * @brief Starts the monitoring with user data.
 * The user data in this case can replace the configured data in the manifest.
 * @param serverUrl
 *           server url.
 * @param appId
 *           application id.
 * @param userData
 *           user custom data (see docs)
 */
+ (BOOL)start:(NSString *)serverUrl
        appId:(NSString *)appId
     userData:(NSDictionary *)userData
        error:(NSError **)error;

/**
 * @function start:settings:error
 * @brief Starts the monitoring using a provided start up settings.
 * @param settings
 *        Startup settings, the startup settings can be used along side a settings
 *        file plist or completely replace the file.
 *        If both the startup settings and the plist file contain the key, the 1
 *        from the startup settings will be used.
 */
+ (BOOL)start:(GLAStartupSettings *)settings error:(NSError **)error;

/*!
 * @function stop
 * @brief Stops Glassbox Monitoring.
 */
+ (void)stop;

/*!
 * @function setScreenAsSensitive
 * @brief Exclude certain screen which is associated via controller from Glassbox Monitoring.
 * @param controller
 *              The controller to exclude from Glassbox Monitoring.
 */
+ (void)setScreenAsSensitive:(UIViewController *)controller;

/*!
 * @function setScreenAsSensitive
 * @brief Exclude certain screen which is associated via controller from Glassbox Monitoring.
 * @param controller
 *              The controller to exclude from Glassbox Monitoring.
 */
+ (void)setScreenAsSensitiveByName:(NSString *)screenName;

/*!
 * Sets view as sensitive. View info is masked during recording.
 * @param view
 *             the view to mark as Sensitive
 */
+ (void)setViewAsSensitive:(UIView *)view;

/**
 * Sets view as sensitive with consideration to visibility flags
 * @param view
 * @param ignoreAlphaChannel
 *          indicates that the view should be masked even if it's alpha channel is set to 0
 * @param ignoreHiddenProperty
 *           indicates that the view should be masked even if it's hidden property is set to true
 * @param ignoreBackgroundControllers
 *           indicates that only the top most UIViewController should be masked
 */
+ (void)setViewAsSensitive:(UIView *)view visibilityFlags:(GLAVisibilityFlags *)visibilityFlags;

/**
 * Removes view sensitivity state. View info is collected.
 */
+ (void)setViewAsVisible:(UIView *)view;

/*!
 * @function setWebViewMonitoring
 * @brief Enable EyeView monitoring and web analytics for the web content which is executed under
 * a certain view.
 * @param view
 *           The view instance that should be monitored.
 */
+(void)adaptView:(UIView *)view;

/**
 * @function reportEvent:event:parameters
 *
 * @brief Reports a user defined event.
 *
 * @discussion The user must at least provide an event name.
 *             in case event is nil the method will return and an event won't be reported.
 * @param event
 *          The event name.
 * @param parameters
 *          event parameters, The parameters dictionary can include any of the primitive wrapper classes
 *          or OBJc collections. complex custom object are not supported.
 */
+ (void)reportEvent:(NSString *)event parameters:(NSDictionary *)parameters;

+ (void)reportError:(NSError *)error;

+ (void)handleOpenURL:(NSURL *)url sourceApplication:(NSString *)sourceApplication annotation:(id)annotation;

+ (void)handleOpenURL:(NSURL *)url options:(NSDictionary *)options;

+ (void)continueUserActivity:(NSUserActivity *)userActivity restorationHandler:(void (^)(NSArray<id<UIUserActivityRestoring>> *restorableObjects))restorationHandler;

+(void)startScreen:(NSString *)screen;

+(void)endScreen;

/**
 *  @function isStarted
 *  @return true if the SDK is currently recording.
 */
+(BOOL)isStarted;

#if CLI_CAPTURE_MODE

/*
 @enum RecordingPermissionStatus
 @abstract
 Constants indicating the client's authorization recording.
 
 @constant RecordingPermissionStatusNotDetermined
 Indicates that the user has not yet made a choice regarding whether the client can record.
 @constant RecordingPermissionStatusAllowed
 The user explicitly allowed the client to record.
 @constant RecordingPermissionStatusDenied
 The user explicitly didn't allow the client to record.
 */
typedef NS_ENUM(NSUInteger, RecordingPermissionStatus) {
    RecordingPermissionStatusNotDetermined = 0,
    RecordingPermissionStatusAllowed = 1,
    RecordingPermissionStatusNotAllowed = 2,
};

@property (class, nonatomic, assign) RecordingPermissionStatus recordingPermissionStatus;

/*!
 * @function requestRecordingPermissionWithCompletionHandler
 * @brief requests the recording permission dialog to show
 * @param granted
 *        tells if the permission was granted or not
 * @param handler
 *        completion handler that will be called after the user chose his decision in the dialog
 */
+ (void)requestRecordingPermissionWithCompletionHandler:(void (^)(BOOL granted))handler;

/*!
 * @function setRequestRecordingPermissionDialogContentWithTitle:message:allowTitle:disallowTitle
 * @brief sets a custom text content in the recording persmission dialog box
 * @param title
 *        dialog title
 * @param message
 *        the text under the title of the dialog
 * @param allowTitle
 *        the text of the allow button
 * @param disallowTitle
 *        the text of the disallow button
 */
+ (void)setRequestRecordingPermissionDialogContentWithTitle:(NSString *)title message:(NSString *)message allowTitle:(NSString *)allowTitle disallowTitle:(NSString *)disallowTitle;

#endif
@end


/*!
 * Possible error codes which can be returned (via the NSError) from the start method.
 */
typedef enum GLAErrorCode {
    GLASSBOX_UNSUPPORTED_OS_VER = 0,
    GLASSBOX_CONFIG_FILE_MISSING,
    GLASSBOX_SERVER_URL_NOT_CONFIGURED,
    GLASSBOX_APPID_INVALID_FORMAT,
    GLASSBOX_NETWORK_ERROR,
    GLASSBOX_INVALID_JSON_FORMAT,
    GLASSBOX_SETTINGS_ERROR
} GLAErrorCode;


