
package com.storyboard;

import android.os.Build;
import android.util.Log;
import android.view.View;
import android.view.ViewGroup;
import android.webkit.WebView;

import com.clarisite.mobile.Glassbox;
import com.clarisite.mobile.VisibilityFlags;
import com.clarisite.mobile.VisibilityFlags.VisibilityFlagsBuilder;
import com.clarisite.mobile.StartupSettings.StartupSettingsBuilder;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;
import com.facebook.react.bridge.Callback;
import com.facebook.react.bridge.ReadableMap;
import com.facebook.react.uimanager.NativeViewHierarchyManager;
import com.facebook.react.uimanager.UIBlock;
import com.facebook.react.uimanager.UIManagerModule;

public class StoryboardReactModule extends ReactContextBaseJavaModule {

  private static final String SDK_TYPE_REACT = "ReactNative";
  private final ReactApplicationContext reactContext;

  public StoryboardReactModule(ReactApplicationContext reactContext) {
    super(reactContext);
    this.reactContext = reactContext;
  }

  @Override
  public String getName() {
    return "Storyboard";
  }

  @ReactMethod
	public void start(String reportUrl, String appId) {
		try {
			Glassbox.start(StartupSettingsBuilder.aSettingsBuilder()
			.withReportUrl(reportUrl)
			.withAppId(appId)
			.withSDKType(SDK_TYPE_REACT)
			//.setInternalConfigurationPolicy()
			.hybridMode()
			.withActivityCtx(getCurrentActivity())
			.build());
		} catch (Exception e) {
			Log.e("Storyboard", "Exception starting Storyboard SDK session recording aborted", e);
		}
	}

	@ReactMethod
	public void stop() {
		try {
			Glassbox.stop();
		} catch (Exception e) {
			Log.d("Storyboard", "Exception when stopping Storyboard SDK");
		}
	}

	@ReactMethod
	public void reportEvent(String eventName, ReadableMap params) {
		try {
			// NOTE: We use the 'toMap' helper method since toHashMap was added to ReadableMap in version 0.46
			Glassbox.reportCustomEvent(eventName, params != null ? ModuleHelpers.toMap(params) : null);
		} catch (Exception e) {
			Log.d("Storyboard", "Exception when trying to report event");
		}
	}

	@ReactMethod
	public void setViewAsSensitive(final int viewId, final ReadableMap params) {
		resolveViewAndExecute(viewId, new OnViewResolved() {
			@Override
			public void onViewResolved(View view) {
				VisibilityFlagsBuilder visibilityFlagsBuilder = VisibilityFlags.builder();
				if (params != null) {
					if (params.hasKey("touchMode")) {
						try{
							visibilityFlagsBuilder.touchMode(VisibilityFlags.TouchMode.valueOf(params.getString("touchMode")));
						}catch(IllegalArgumentException e){
							Log.e("Storyboard", "Failed to parse VisibilityFlags.TouchMode", e);
						}
					}
					if (params.hasKey("omitAnalytics") && params.getBoolean("omitAnalytics")) {
						visibilityFlagsBuilder.omitAnalytics();
					}
				}

				VisibilityFlags visibilityFlags = visibilityFlagsBuilder.build();
				Log.d("Storyboard", "visibilityFlags=" + visibilityFlags.getTouchState() + ", omitAnalytics=" + visibilityFlags.isOmitAnalytics());
				Glassbox.setViewAsSensitive(view, visibilityFlags);
			}
		});
	}

	@ReactMethod
	public void disableViewHiding(final int viewId) {
		resolveViewAndExecute(viewId, new OnViewResolved() {
			@Override
			public void onViewResolved(View view) {
				Glassbox.disableViewSensitivity(view);
			}
		});
	}

	@ReactMethod
	public void setViewTag(final int viewId, final String tag) {
		resolveViewAndExecute(viewId, new OnViewResolved() {
			@Override
			public void onViewResolved(View view) {
				Glassbox.getUserTagManager().setTag(view, tag);
			}
		});
	}

	@ReactMethod
	public void startScreen(final String screenName) {
		Glassbox.startScreen(screenName);
	}

	@ReactMethod
	public void endScreen() {
		Glassbox.endScreen();
	}

	@ReactMethod
	public void trackView(final int viewId) {
		resolveViewAndExecute(viewId, new OnViewResolved() {
			@Override
			public void onViewResolved(View view) {
				try{

					if(view instanceof ViewGroup && !(view instanceof WebView)){
						ViewGroup viewGroup = (ViewGroup) view;
						for(int i=0;i<viewGroup.getChildCount();i++){
							if(viewGroup.getChildAt(i) instanceof WebView){
								view = viewGroup.getChildAt(i);
								break;
							}
						}
					}

					WebView webView = (WebView) view;
					webView.getSettings().setJavaScriptEnabled(true);
					//DEBUG: Uncomment the following 5 lines for debugging 
					// webView.getSettings().setAllowUniversalAccessFromFileURLs(true);
					// webView.getSettings().setAllowFileAccessFromFileURLs(true);
					// if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.KITKAT) {
					// 	WebView.setWebContentsDebuggingEnabled(true);
					// }
					Glassbox.trackView(view);
				} catch(Exception e){
					Log.e("Storyboard", "Exception when trying to track view " + viewId, e);
				}
			}
		});
	}

	private void resolveViewAndExecute(final int viewId, final OnViewResolved action) {
		UIManagerModule uiManager = getReactApplicationContext().getNativeModule(UIManagerModule.class);
		if (uiManager == null) {
			return;
		}

		uiManager.addUIBlock(new UIBlock() {
			@Override
			public void execute(NativeViewHierarchyManager nativeViewHierarchyManager) {
				try {
					View view = nativeViewHierarchyManager.resolveView(viewId);
					if (view != null) {
						action.onViewResolved(view);
					} else {
						Log.d("Storyboard", "Could not find view with id " + viewId);
					}
				} catch (Exception e) {
					Log.d("Storyboard", "Exception when trying to locate view " + viewId);
				}
			}
		});
	}

	interface OnViewResolved {
		void onViewResolved(View view);
	}
}